const std = @import("std");

const InteractionError = error{
    TaskNotFound,
};

pub fn userInteraction(allocator: std.mem.Allocator, tasks: *std.ArrayList([]const u8)) !void {
    const stdout = std.io.getStdOut().writer();

    while (true) {
        try stdout.print("Please enter your choice add/remove/edit/quit: ", .{});
        const input = getInput(allocator) catch |err| {
            std.debug.print("Error acquiring input:\n{}\n", .{err});
            continue;
        };
        defer allocator.free(input);
        std.debug.print("You entered: {s}\n", .{input});

        const Case = enum { add, edit, remove, quit };
        const case = std.meta.stringToEnum(Case, input) orelse return;

        switch (case) {
            .add => {
                try stdout.print("Please enter task name: ", .{});
                const task_name = getInput(allocator) catch |err| {
                    std.debug.print("Error acquiring input:\n{}\n", .{err});
                    continue;
                };
                defer allocator.free(task_name);
                const duped = try allocator.dupe(u8, task_name);
                tasks.append(duped) catch |err| {
                    std.debug.print("Failed to add the specified task:\n{}\n", .{err});
                };
            },
            .edit => {
                try stdout.print("Please enter task name: ", .{});
                const task_name = getInput(allocator) catch |err| {
                    std.debug.print("Error acquiring input:\n{}\n", .{err});
                    continue;
                };
                defer allocator.free(task_name);

                const id = getTaskIndex(tasks, task_name) catch {
                    std.debug.print("Invalid task name...", .{});
                    continue;
                };
                try stdout.print("Please enter new task name: ", .{});
                const new_task_name = try getInput(allocator);
                defer allocator.free(new_task_name);

                const new_task_item = try allocator.dupe(u8, new_task_name);
                // free existing memory alloc
                allocator.free(tasks.items[id]);
                tasks.items[id] = new_task_item;
            },
            .remove => {
                try stdout.print("Please enter task name: ", .{});
                const task_name = getInput(allocator) catch |err| {
                    std.debug.print("Error acquiring input:\n{}\n", .{err});
                    continue;
                };
                defer allocator.free(task_name);

                const id = getTaskIndex(tasks, task_name) catch {
                    std.debug.print("Invalid task name...", .{});
                    continue;
                };

                // free existing memory alloc
                allocator.free(tasks.items[id]);
                _ = tasks.orderedRemove(id);
            },
            .quit => break,
        }
    }
}

fn getInput(allocator: std.mem.Allocator) ![]const u8 {
    const stdin = std.io.getStdIn().reader();
    const bare_line = try stdin.readUntilDelimiterAlloc(allocator, '\n', 8192);
    // defer std.heap.page_allocator.free(bare_line);
    // windows legacy
    return std.mem.trim(u8, bare_line, "\r");
}

fn getTaskIndex(tasks: *std.ArrayList([]const u8), task_name: []const u8) !usize {
    for (tasks.items, 0..) |value, i| {
        if (std.mem.eql(u8, task_name, value)) {
            return i;
        }
    }

    return InteractionError.TaskNotFound;
}
